<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class RescisaoComplementarController extends Crud {

    protected $calculoFolhaService;
    protected $dataSource = 'fpa_formresccomplementar';
    protected $dataSourceGet = 'getFormResComp';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERADOR');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        parent::__construct($dataSourceManager);
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
    }

    public function getFormResComp (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $rows = $dataSet->getRows();

        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            } 
            
            if($row['DSORDEM'] == 'LISTA_LOTE'){
                $row['NRLOTEVINC_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRLOTEVINC_LIST'] as $chave => $value)
                    $row['NRLOTEVINC_LIST'][$chave] = intval($value);
                
                $row['LIST_NMLOTEVINC'] = explode(",", $row['LIST_NMLOTEVINC']);
            }   
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }   
        }
        $response->addDataSet(new DataSet($this->dataSourceGet, $rows));
    }

    public function setFormResComp (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSource;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if (array_key_exists('NRFORMRESCISAO', $row)) {
                    if ($row['NRFORMRESCISAO'] === '') {
                        $row['NRFORMRESCISAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMRESCCOMPLEMENTAR', 1, 12, true);
                    }
                } else {
                    $row['NRFORMRESCISAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FORMRESCCOMPLEMENTAR', 1, 12, true);
                }
                $row['CDOPERADOR'] = $row['CDOPERULTATU'];
                $row['IDEXECUTAPREP'] = 'N';
                
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
 
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_LOTE"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(count($row['DSFIM']) > 0 && is_array($row['DSFIM'])){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFormResComp (DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = $this->dataSource;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}