<?php

namespace HCM\FPA\ObrigacaoLegal\Manad;

use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Bloco0;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\BlocoI;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\BlocoL;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Bloco9;
use HCM\FPA\ObrigacaoLegal\Manad\Blocos\Registros\Registro0000;
use HCM\FPA\ObrigacaoLegal\Manad\QueryManad;
use HCM\FPA\ObrigacaoLegal\Manad\Manad;
use HCM\FPA\Entity\FpaManadmagnetEntity;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\Repositories;

class PercorreManad {

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_dtCompetencia;
    private $_mesCompetencia;
    private $_manad;
    private $_objMagnetico;
    private $_bloco0;
    private $_blocoI;
    private $_blocoL;
    private $_bloco9;
    private $queryManad;
    private $qtdTpRegK;
    private $qtdRegK;
    private $qtdReg9;
    private $qtdLin;
    
    const ql_msg = '<br>'; //Quebra de linha para as mensagens
    
    public function __construct($entityManager, $hcmParameters, $dataSet){
        $this->entityManager   = $entityManager;
        $this->hcmParameters   = $hcmParameters;
        $this->_nrOrg          = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao    = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador     = $hcmParameters->getCdOperador();
        $this->_dtCompetencia  = $hcmParameters->getDtCompetencia();
        $this->_mesCompetencia = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        
        $this->_inicializaVariaveis($dataSet);

        $this->_objMagnetico = new FpaManadmagnetEntity($this->entityManager, $this->hcmParameters, $this->_nrOrg, $this->_mesCompetencia, $this->_nrInscEstrutResp);

        $tp_processamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch (strtoupper($tp_processamento)){
            case FPA::PROC_COMPETENCIA_MENSAL :
                $this->_retrocesso = false;
                break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true;
                break;
            default:
                throw new \Exception('Tipo de processamento '.$tp_processamento.' inválido');
        }
    }

    private function _inicializaVariaveis($dataSet) {
        $this->_bloco0 = new Bloco0();
        $this->_blocoI = new BlocoI();
        $this->_blocoL = new BlocoL();
        $this->_bloco9 = new Bloco9();

        $this->queryManad = new QueryManad($this->entityManager,$this->hcmParameters);

        $this->_buscaDadosFormulario($dataSet);

        $estruturaResp = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura($this->_nrOrg, $this->_nrEstrResponsa, $this->_mesCompetencia);
        $this->_nrInscEstrutResp = $estruturaResp->historico->getCdcnpjestrut();
                        
        $this->beneficiariosManad = [];
        $this->registrosMagnetico = [];
        $this->dataEstrutResp = [];
        $this->qtdTpRegK[Registro0000::REGISTRO_K001] = 1; //Adiciona a totalização do registro inicial K001.
        $this->qtdRegK = 2; //K001 e K990.
        $this->qtdReg9 = 0;
        $this->qtdLin  = 0;
    }
    
    //Função que carrega os dados do formulário. Na falha esta seta o nrOrg para -1 indicando a falha
    private function _buscaDadosFormulario($dataSet){
        
        $formObj = $this->entityManager->getRepository(Repositories::FPA_FORMMANAD)->findOneBy(array("nrorg" => $dataSet['NRORG'], "nrformmanad"=>$dataSet['NRFORMMANAD']));
        
        //Dados do formulário
        if(is_object($formObj)) {   //Somente grava os dados se encontrou o formulário segundo a Org e seu Nro
            $this->_nrForm         = $formObj->getNrformmanad();
            $this->_nmForm         = $formObj->getNmformmanad();
            $this->_idAtivo        = $formObj->getIdativo();
            $this->_dsOrdem        = $formObj->getDsordem();
            $this->_nomeInicial    = $formObj->getNomeinicial();
            $this->_nomeFinal      = $formObj->getNomefinal();
            $this->_dsInicio       = $formObj->getNomeinicial();
            $this->_dsFim          = $formObj->getNomefinal();
            $this->_idCentrEscrit  = $formObj->getIdcentrescrit();
            $this->_idFinaldArq    = $formObj->getIdfinaldarq();
            $this->_idGeraMagnetic = $formObj->getIdgerameiomag();
            $this->_idGeraArqv     = $formObj->getIdgeraarqv();
            $this->_nrPessResponsa = $formObj->getNrpessoarespons();
            $this->_nrEstrResponsa = $formObj->getNrestrutrespons();
            $this->_idTpInscriResp = $formObj->getIdtpinscrespons();
            $this->_nrGrupoInciden = $formObj->getNrgrpincide();
            $this->_dtPeriodIni    = $formObj->getDtperiodini();
            $this->_dtPeriodFim    = $formObj->getDtperiodfim();
            $this->_dtServInfoIni  = $formObj->getDtservinfoini();
            $this->_dtServInfoFim  = $formObj->getDtservinfofim();
            $this->_nrEstrutuLegal = $formObj->getNrestrutlegal();
            $this->_nrEstrutuGeren = $formObj->getNrestrutgeren();
            $this->_cdIniMatricula = $formObj->getCdinimatricula();
            $this->_cdFimMatricula = $formObj->getCdfimmatricula();
        } else {
            $this->_nrOrg = -1; //indica que não carregou os dados
            $this->msgErro = 'Formulário não carregado. O número do formulário '.$dataSet['NRFORMMANAD'].' da organização número '.$dataSet['NRORG'].' não foi encontrado.'.self::ql_msg;
        }
    }
    
    public function percorre(){
        if ($this->_retrocesso === false) {
            $resultado = $this->_geraMANAD();
            return $resultado;
        } else {
            $cont = $this->_objMagnetico->excluirCompetencia();

            if($cont == 0) {
                throw new \Exception("Não há registro de magnético a retroceder da Organização ".$this->_nrOrg." na Competência ".$this->_dtCompetencia." e CNPJ da Estrutura do Responsável ".$this->_nrInscEstrutResp);
            }
            return true;
        }
    }

    private function _geraMANAD(){ 
        if ($this->_idGeraMagnetic == 'N' && $this->_idGeraArqv == 'N')
            throw new \Exception('Não há dados para gerar. Selecione no formulário a geração de magnético e/ou arquivo.');

        $this->_preparaBloco0();
        $this->_preparaBlocosIeL(); //Blocos sem dados podem ser gerados antes.

        if ($this->_idGeraMagnetic == 'S') {
            $registros = $this->_objMagnetico->qdeRegistros();

            if ($registros == 0) {
                $this->_objMagnetico->addRegCabecal($this->_nrInscEstrutResp, $this->_nrPessResponsa, 'S');

                $vinculos = $this->retornaVinculosManad();

                if(empty($vinculos)) throw new \Exception('Não foram encontrados vínculos para o critério de seleção aplicado.');
                                
                foreach($vinculos as $vinculo) {
                    $beneficiarios[] = $vinculo['NRPESSOA'];
                }

                $this->_manad = new Manad($this->entityManager, $this->hcmParameters, $this->_objMagnetico, $this->dataEstrutResp, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim,
                                          $this->_dtPeriodIni,  $this->_dtPeriodFim);
                
                foreach($beneficiarios as $nrpessoa) {
                    $this->_manad->geraManad($nrpessoa);
                }
                
                $this->_manad->_preparaRegistroK150(); //Registro geral fora de vínculo.
            } else {
                throw new \Exception("Já existe magnético do MANAD da Organização ".$this->_nrOrg." na Competência ".$this->_dtCompetencia." e CNPJ da Estrutura do Responsável ".$this->_nrInscEstrutResp.".".self::ql_msg."Favor retroceder.");
            }
        }
        
        $this->setRegistrosMagnetico();      
        
        if ($this->_idGeraArqv == 'S')
            $arquivo = $this->_geraMagnetico();
        
        return ['ARQUIVO'     => $arquivo ?? NULL,
                'SOMAGNETICO' => !isset($arquivo) ? true : false,
                'MSGS'        => $this->getMensagens()];
    }

    private function _geraMagnetico(){
        $nomeArquivo = "MANAD_".$this->_nrInscEstrutResp.".txt";

        if(!isset($this->_manad)) {
            $this->_manad = new Manad($this->entityManager, $this->hcmParameters, $this->_objMagnetico, $this->dataEstrutResp, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim,
                                      $this->_dtPeriodIni,  $this->_dtPeriodFim);
        }

        $this->_bloco0->_registro0000->geraRegistro0000();
        $this->escreveArquivo($nomeArquivo, $this->_bloco0->_registro0000->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF, false, true);

        $this->_bloco0->_registro0001->geraRegistro0001(); 
        $this->escreveArquivo($nomeArquivo, $this->_bloco0->_registro0001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);

        $this->_bloco0->_registro0050->geraRegistro0050();
        $this->escreveArquivo($nomeArquivo, $this->_bloco0->_registro0050->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);

        $this->_bloco0->_registro0100->geraRegistro0100();
        $this->escreveArquivo($nomeArquivo, $this->_bloco0->_registro0100->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);

        $this->_bloco0->_registro0990->geraRegistro0990();
        $this->escreveArquivo($nomeArquivo, $this->_bloco0->_registro0990->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);

        $this->escreveArquivo($nomeArquivo, $this->_blocoI->_registroI001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
        $this->escreveArquivo($nomeArquivo, $this->_blocoI->_registroI990->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
        $this->escreveArquivo($nomeArquivo, $this->_manad->_geraERetornaRegistroK001()); //Abertura Bloco K.

        foreach($this->registrosMagnetico as $tpReg) {
            $gravouQtdReg = false;

            foreach($tpReg as $linha) {
                if(!$gravouQtdReg) {
                    $this->qtdTpRegK[$linha->getIdtiporegistro()] = count($tpReg);
                    $gravouQtdReg = true;
                }
                $this->escreveArquivo($nomeArquivo, $linha->getDslinharegistro().Registro0000::QUEBRA_LINHA_CR_LF);
            }
        }
        
        $this->escreveArquivo($nomeArquivo, $this->_manad->_geraERetornaRegistroK990($this->qtdRegK)); //Finalização Bloco K.
        $this->qtdTpRegK[Registro0000::REGISTRO_K990] = 1; //Adiciona a totalização do registro final K990.

        $this->escreveArquivo($nomeArquivo, $this->_blocoL->_registroL001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
        $this->escreveArquivo($nomeArquivo, $this->_blocoL->_registroL990->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);

        $this->_preparaBloco9($nomeArquivo);

        return $nomeArquivo;
    }

    private function _preparaBloco0(){
        $this->_preparaRegistro0000();
        $this->_preparaRegistro0050();
        $this->_preparaRegistro0100();
    }

    private function _preparaBlocosIeL(){
        $this->_blocoI->_registroI001->geraRegistroI001();
        $this->_blocoI->_registroI990->geraRegistroI990();

        $this->_blocoL->_registroL001->geraRegistroL001();
        $this->_blocoL->_registroL990->geraRegistroL990();
    }

    private function _preparaBloco9($nomeArquivo){
        $this->_preparaRegistro9001($nomeArquivo);
        $this->_preparaRegistro9900($nomeArquivo);
        $this->_preparaRegistro9990($nomeArquivo);
        $this->_preparaRegistro9999($nomeArquivo);
    }

    private function _preparaRegistro0000(){
        $cpf = null; $cnpj = null;

        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia];
            
            $sqlResult = $this->queryManad->get_reg0000($params);
            $this->dataEstrutResp = $sqlResult;
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0000, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0000, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NOMEEMPRESARIAL'])) {
                $this->msgErro .= "Está faltando o nome empresarial do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDUF'])) {
                $this->msgErro .= "Está faltando a unidade da federação do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDMUN'])) {
                $this->msgErro .= "Está faltando Código do município do domicílio fiscal do contribuinte.".self::ql_msg;
            }
            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cpf ou cnpj do contribuinte.".self::ql_msg;
            }
            else {
                if($sqlResult['TPINSC'] == 'CNPJ')
                    $cnpj = $sqlResult['CDCNPJ'];
                else
                    $cpf = substr($sqlResult['CDCNPJ'], 0, 11);
            }
            if(empty($this->_dtPeriodIni) || empty($this->_dtPeriodFim)) {
                $this->msgErro .= "Está faltando o período inicial ou final.".self::ql_msg;
            }
            if(is_null($this->_idCentrEscrit)){
                $this->msgErro .= "Está faltando a indicação da Centralização de Escrituração.".self::ql_msg;
            }
            if(empty($this->_idFinaldArq)){
                $this->msgErro .= "Está faltando a indicação da Finalidade do Arquivo.".self::ql_msg;
            }
            
            $nome = $sqlResult['NOMEEMPRESARIAL'];
            $cei  = $sqlResult['CDCEI'];
            $dtIni = $this->_dtPeriodIni->format('dmY');
            $dtFim = $this->_dtPeriodFim->format('dmY');
            $uf = $sqlResult['CDUF'];
            $codMun = $sqlResult['CDMUN'];
                
            $this->_bloco0->_registro0000->setNome($nome);
            $this->_bloco0->_registro0000->setCnpj($cnpj);
            $this->_bloco0->_registro0000->setCpf($cpf);
            $this->_bloco0->_registro0000->setCei($cei);
            $this->_bloco0->_registro0000->setNit(null);
            $this->_bloco0->_registro0000->setUf($uf);
            $this->_bloco0->_registro0000->setIe(null);
            $this->_bloco0->_registro0000->setCodMun($codMun);
            $this->_bloco0->_registro0000->setIm(null);
            $this->_bloco0->_registro0000->setSuframa(null);
            $this->_bloco0->_registro0000->setIndCentr($this->_idCentrEscrit);
            $this->_bloco0->_registro0000->setDtIni($dtIni);
            $this->_bloco0->_registro0000->setDtFim($dtFim);
            $this->_bloco0->_registro0000->setCodFin($this->_idFinaldArq);
        }
    }

    private function _preparaRegistro0050(){
        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia,
                       'P_NRORGPADRAO'            => $this->_nrOrgPadrao,
                       'P_NRPESSRESPONSA'         => $this->_nrPessResponsa];

            $sqlResult = $this->queryManad->get_reg0050($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0050, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0050, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NMNOMERESP'])) {
                $this->msgErro .= "Está faltando o nome do responsável contabilista/escritório de contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cnpj do escritório de contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['CDCPF'])) {
                $this->msgErro .= "Está faltando o cpf do responsável contabilista.".self::ql_msg;
            }
            if(empty($sqlResult['CDCRC'])) {
                $this->msgErro .= "Está faltando o número de inscrição do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['DSEND'])) {
                $this->msgErro .= "Está faltando o endereço do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['CDCEP'])) {
                $this->msgErro .= "Está faltando o CEP do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['SGESTADO'])) {
                $this->msgErro .= "Está faltando a UF do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            if(empty($sqlResult['NRTELEFONE'])) {
                $this->msgErro .= "Está faltando o telefone do responsável contabilista no Conselho Regional de Contabilidade.".self::ql_msg;
            }
            
            $nome = $sqlResult['NMNOMERESP'];
            $cnpj = $sqlResult['CDCNPJ'];
            $cpf  = $sqlResult['CDCPF'];
            $crc  = $sqlResult['CDCRC'];
            $dtIni = $this->_dtPeriodIni->format('dmY');
            $dtFim = $this->_dtPeriodFim->format('dmY');
            $end = $sqlResult['DSEND'];
            $num = $sqlResult['NRNUMEND'];
            $compl  = $sqlResult['DSCOMP'];
            $bairro = $sqlResult['DSBAIRRO'];
            $cep = $sqlResult['CDCEP'];
            $uf  = $sqlResult['SGESTADO'];
            $fone = trim($sqlResult['NRDDD'].' '.$sqlResult['NRTELEFONE'], ' ');
            $fax  = trim($sqlResult['NRDDDFAX'].' '.$sqlResult['NRFAX'], ' ');
            $email = $sqlResult['NMEMAIL'];
                
            $this->_bloco0->_registro0050->setNome($nome);
            $this->_bloco0->_registro0050->setCnpj($cnpj);
            $this->_bloco0->_registro0050->setCpf($cpf);
            $this->_bloco0->_registro0050->setCrc($crc);
            $this->_bloco0->_registro0050->setDtIni($dtIni);
            $this->_bloco0->_registro0050->setDtFim($dtFim);
            $this->_bloco0->_registro0050->setEnd($end);
            $this->_bloco0->_registro0050->setNum($num);
            $this->_bloco0->_registro0050->setCompl($compl);
            $this->_bloco0->_registro0050->setBairro($bairro);
            $this->_bloco0->_registro0050->setCep($cep);
            $this->_bloco0->_registro0050->setUf($uf);
            $this->_bloco0->_registro0050->setFone($fone);
            $this->_bloco0->_registro0050->setFax($fax);
            $this->_bloco0->_registro0050->setEmail($email);
        }
    }

    private function _preparaRegistro0100(){
        try{
            $params = ['P_NRORG'                  => $this->_nrOrg,
                       'P_ESTRUTURARESPONSAVEL'   => $this->_nrEstrResponsa,
                       'P_DTMESCOMPETENC'         => $this->_dtCompetencia,
                       'P_NRPESSRESPONSA'         => $this->_nrPessResponsa];

            $sqlResult = $this->queryManad->get_reg0100($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query: Bloco 0, Registro tipo 0100, item 4.2.'.self::ql_msg.'Error:'.$e->getMessage());
        }

        if(empty($sqlResult)) {
            $this->msgErro .= "Sem dados no Bloco 0, Registro tipo 0100, item 4.2.".self::ql_msg;
        } else {
            if(empty($sqlResult['NMNOMERESP'])) {
                $this->msgErro .= "Está faltando o nome da empresa/técnico responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($sqlResult['NMCARGO'])) {
                $this->msgErro .= "Está faltando o cargo do responsável na empresa.".self::ql_msg;
            }

            if(empty($sqlResult['CDCNPJ'])) {
                $this->msgErro .= "Está faltando o cnpj da empresa responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($sqlResult['CDCPF'])) {
                $this->msgErro .= "Está faltando o cpf do técnico responsável pela geração do arquivo.".self::ql_msg;
            }
            if(empty($this->_dtServInfoIni) || empty($this->_dtServInfoFim)) {
                $this->msgErro .= "Está faltando o período da prestação do serviço de informática.".self::ql_msg;
            }
            
            $nome  = $sqlResult['NMNOMERESP'];
            $cargo = $sqlResult['NMCARGO'];
            $dtIni = $this->_dtServInfoIni->format('dmY');
            $dtFim = $this->_dtServInfoFim->format('dmY');
            $cnpj = $sqlResult['CDCNPJ'];
            $cpf  = $sqlResult['CDCPF'];
            $fone = trim($sqlResult['NRDDD'].' '.$sqlResult['NRTELEFONE'], ' ');
            $fax  = trim($sqlResult['NRDDDFAX'].' '.$sqlResult['NRFAX'], ' ');
            $email = $sqlResult['NMEMAIL'];
                
            $this->_bloco0->_registro0100->setNome($nome);
            $this->_bloco0->_registro0100->setCargo($cargo);
            $this->_bloco0->_registro0100->setDtIni($dtIni);
            $this->_bloco0->_registro0100->setDtFim($dtFim);
            $this->_bloco0->_registro0100->setCnpj($cnpj);
            $this->_bloco0->_registro0100->setCpf($cpf);
            $this->_bloco0->_registro0100->setFone($fone);
            $this->_bloco0->_registro0100->setFax($fax);
            $this->_bloco0->_registro0100->setEmail($email);
        }
    }

    private function _preparaRegistro9001($nomeArquivo){
        $this->_bloco9->_registro9001->geraRegistro9001();
        $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9001->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
        $this->qtdReg9++;
    }

    private function _preparaRegistro9900($nomeArquivo){
        $this->totalizaBlocos0eI($nomeArquivo);
        $this->totalizaBlocoK($nomeArquivo);
        $this->totalizaBlocoL($nomeArquivo);
        $this->totalizaBloco9AteReg9900($nomeArquivo);
    }

    private function _preparaRegistro9990($nomeArquivo){
        $this->_bloco9->_registro9990->setQtdLin9($this->qtdReg9 + 2); //+ o próprio 9990 e 9999.
        $this->_bloco9->_registro9990->geraRegistro9990();
        $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9990->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
    }

    private function _preparaRegistro9999($nomeArquivo){
        $this->_bloco9->_registro9999->setQtdLin($this->qtdLin + 1); //+ o próprio 9999.
        $this->_bloco9->_registro9999->geraRegistro9999();
        $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9999->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
    }

    private function totalizaBlocos0eI($nomeArquivo) {
        $regComUmaPosicao = [
            Registro0000::REGISTRO_0000,
            Registro0000::REGISTRO_0001,
            Registro0000::REGISTRO_0050,
            Registro0000::REGISTRO_0100,
            Registro0000::REGISTRO_0990,
            Registro0000::REGISTRO_I001,
            Registro0000::REGISTRO_I990
        ];

        foreach($regComUmaPosicao as $tpReg) {
            $this->qtdReg9++;
            $this->_bloco9->_registro9900->setTpReg($tpReg);
            $this->_bloco9->_registro9900->setQtdReg(1);
            
            $this->_bloco9->_registro9900->geraRegistro9900();
            $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9900->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
            
            $this->_bloco9->_registro9900->setLinhaRegistro();
        }
    }

    private function totalizaBlocoK($nomeArquivo) {
        foreach($this->qtdTpRegK as $tpReg => $qtd) {
            $this->qtdReg9++;
            $this->_bloco9->_registro9900->setTpReg($tpReg);
            $this->_bloco9->_registro9900->setQtdReg($qtd);
            
            $this->_bloco9->_registro9900->geraRegistro9900();
            $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9900->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
            
            $this->_bloco9->_registro9900->setLinhaRegistro();
        }
    }

    private function totalizaBlocoL($nomeArquivo) {
        $regComUmaPosicao = [
            Registro0000::REGISTRO_L001,
            Registro0000::REGISTRO_L990
        ];

        foreach($regComUmaPosicao as $tpReg) {
            $this->qtdReg9++;
            $this->_bloco9->_registro9900->setTpReg($tpReg);
            $this->_bloco9->_registro9900->setQtdReg(1);
            
            $this->_bloco9->_registro9900->geraRegistro9900();
            $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9900->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
            
            $this->_bloco9->_registro9900->setLinhaRegistro();
        }
    }

    private function totalizaBloco9AteReg9900($nomeArquivo) {
        $regComUmaPosicao = [
            Registro0000::REGISTRO_9001,
            Registro0000::REGISTRO_9900,
            Registro0000::REGISTRO_9990,
            Registro0000::REGISTRO_9999
        ];
        
        foreach($regComUmaPosicao as $tpReg) {
            $qtd = ($tpReg == Registro0000::REGISTRO_9900) ? ($this->qtdReg9 + 2) : 1; //Totaliza o registro '9900|9900|' considerando o valor atual + 9990 e 9999. Para os demais valor 1.
            
            $this->_bloco9->_registro9900->setTpReg($tpReg);
            $this->_bloco9->_registro9900->setQtdReg($qtd);
            $this->_bloco9->_registro9900->geraRegistro9900();
            $this->escreveArquivo($nomeArquivo, $this->_bloco9->_registro9900->getLinhaRegistro().Registro0000::QUEBRA_LINHA_CR_LF);
            $this->_bloco9->_registro9900->setLinhaRegistro();
            $this->qtdReg9++;
        }
    }
    
    private function retornaVinculosManad() {
        $dtUltDiaMes = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($this->_mesCompetencia));
        $primeiroDiaAno = DateUtil::getPrimeiroDiaDoAno(DateUtil::getDataDeString($this->_mesCompetencia));
        $primeiroDiaAno = DateUtil::addSubIntervalo($primeiroDiaAno, -1, DateUtil::MES);
        
        $params = ['P_DS_INICIO'       => $this->_dsInicio,
                   'P_DS_FIM'          => $this->_dsFim,
                   'P_DSORDEM'         => $this->_dsOrdem,
                   'P_PRIMEIRODIAANO'  => $primeiroDiaAno->format('d/m/Y'),
                   'P_DTULTDIAMES'     => $dtUltDiaMes->format('d/m/Y'),
                   'P_NRORG'           => $this->_nrOrg];
        
        try{
            return $this->queryManad->montaSQLVinculoManad($params);
        } catch(\Exception $e){
            throw new \Exception('Erro na query montaSQLVinculoManad: Bloco K - Folha de Pagamento.'.self::ql_msg.'Error:'.$e->getMessage());
        }
    }

    public function getMensagens(){
        return is_null($this->msgErro) ? false : $this->msgErro;
    }
    
    private function setRegistrosMagnetico(){
        $registros = $this->_objMagnetico->retornaRegistros();
        $count = count($registros);

        if($count == 0)
            throw new \Exception("Não há registro de magnético da Organização ".$this->_nrOrg." na Competência ".$this->_dtCompetencia." e CNPJ da Estrutura do Responsável ".$this->_nrInscEstrutResp);

        $this->qtdRegK += $count;

        foreach($registros as $registro) {
            $this->registrosMagnetico[$registro->getIdtiporegistro()][] = $registro;
        }
    }

    private function escreveArquivo($nomeArquivo, $conteudo, $array = false, $limpaArquivo = false){
        $this->qtdLin++;
        FPA::escreveArquivo($nomeArquivo, $conteudo, $array, $limpaArquivo);
    }
}