<?php
namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager; 
use HCM\PTO\Repository\ApuracaoFrequenciaRepository;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\AlertException;
use HCM\Geral\Services\HcmServices;

class ApuracaoFrequenciaService {

    /**
     * @var apuracaoFrequenciaRepository
     */
    protected $apuracaoFrequenciaRepository;
    
    // Tempos mínimos de intervalos
    const TEMPO_JORNADA_INTERVALO_15_TEMPO_MAX  = 6;
    const TEMPO_JORNADA_INTERVALO_15_TEMPO_MIN  = 4;
    
    // Constants usadas para validação
    const APUR_TEMPO_MAXIMO_SEM_ALMOCO          = 6;
    const APUR_TEMPO_MINIMO_ALMOCO              = 1;
    const APUR_TEMPO_MIN_SEM_ALMOCO             = 4;
    
    // Constants HE
    const TIPO_1AS = '1AS';
    const TIPO_SEG = 'SEG';

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, HcmServices $hcmServices, ApuracaoFrequenciaRepository $apuracaoFrequenciaRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->hcmServices = $hcmServices;
        $this->apuracaoFrequenciaRepository = $apuracaoFrequenciaRepository;
    }

    /**
     * @return int
     */    
    public function getVinculosPorSelecaoNRescindidos ($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrOrgPadrao, $nrOrg, $apuraRescindidos, $fimPeriodo, $iniPeriodo) {
        $result = $this->apuracaoFrequenciaRepository->getVinculosPorSelecaoNRescindidos($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrOrgPadrao, $nrOrg, $apuraRescindidos, $fimPeriodo, $iniPeriodo);
        return $result;
    }

    public function getResumoSituFreqVinculo ($nrOrg, $nrVinculom, $nrPeriododApuracao) {
        $result = $this->apuracaoFrequenciaRepository->getResumoSituFreqVinculo($nrOrg, $nrVinculom, $nrPeriododApuracao);
        return $result;
    }

    public function verificaControleJornada ($nrOrg, $nrvinculom, $fimPeriodo) {
        $result = $this->apuracaoFrequenciaRepository->verificaControleJornada($nrOrg, $nrvinculom, $fimPeriodo);
        return $result;
    }
    
    public function retrocessoApuracao($nrOrg, $nrVinculo, $nrPeriodoApuracao) {
        $this->deleteMarcacoes($nrOrg, $nrVinculo, $nrPeriodoApuracao);
        $this->deleteIntervalos($nrOrg, $nrVinculo, $nrPeriodoApuracao);
        $this->deleteResumos($nrOrg, $nrVinculo, $nrPeriodoApuracao);

        $this->updateMarcacoesAlteradas($nrOrg, $nrVinculo, $nrPeriodoApuracao);

        $this->updateNovoCodigoResumos();
        $this->updateNovoCodigoMarcacoes();
        $this->updateNovoCodigoIntervalos();
    }

    public function deleteMarcacoes ($nrorg, $nrvinculom, $nrperiodoapuracao) {
        $result = $this->apuracaoFrequenciaRepository->deleteMarcacoes($nrorg, $nrvinculom, $nrperiodoapuracao);
    }

    public function deleteIntervalos ($nrorg, $nrvinculom, $nrperiodoapuracao) {
        $result = $this->apuracaoFrequenciaRepository->deleteIntervalos($nrorg, $nrvinculom, $nrperiodoapuracao);
    }

    public function deleteResumos ($nrorg, $nrvinculom, $nrperiodoapuracao) {
        $result = $this->apuracaoFrequenciaRepository->deleteResumos($nrorg, $nrvinculom, $nrperiodoapuracao);
    }

    public function updateMarcacoesAlteradas ($nrorg, $nrvinculom, $nrperiodoapuracao) {
        return $this->apuracaoFrequenciaRepository->updateMarcacoesAlteradas($nrorg, $nrvinculom, $nrperiodoapuracao);
    }

    public function updateNovoCodigoResumos () {
        return $this->apuracaoFrequenciaRepository->updateNovoCodigoResumos();
    }

    public function updateNovoCodigoMarcacoes () {
        return $this->apuracaoFrequenciaRepository->updateNovoCodigoMarcacoes();
    }

    public function updateNovoCodigoIntervalos () {
        return $this->apuracaoFrequenciaRepository->updateNovoCodigoIntervalos();
    }

    public function getPeriodoSimulacao($parameters) {
        return $this->apuracaoFrequenciaRepository->getPeriodoSimulacao($parameters);
    }

    public function getVinculosSimulacao($parameters) {
        return $this->apuracaoFrequenciaRepository->getVinculosSimulacao($parameters['CRITERIO'], $parameters['DSINICIO'], $parameters['DSFIM'], $parameters['DTMESCOMPETENC'], $parameters['NRTIPOVINCULOM'], $parameters['NRORGPADRAO'], $parameters['NRORG']);
    }
    
    public function requestApuracaoWebService($nrVnculo, $nrPeriodoApuracao) {
        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVnculo,
                'NRPERIODOAPURACAO' => $nrPeriodoApuracao
            ]
        ];
        
        $response = $this->hcmServices->send('apurarPeriodo', $params, $this->hcmParameters->getNrorgTrabalho());
        $alerts = $response['dataset']['info'] ?? null;
        if (!empty($alerts)) {
            throw new AlertException($this->createApuracaoRequestAlertMessage($alerts));
        }
        
        return $response['dataset']['apurarPeriodo']['apurado'] ?? false;
    }
    
    public function requestRetrocessoApuracaoWebService($nrVnculo, $nrPeriodoApuracao) {
        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVnculo,
                'NRPERIODOAPURACAO' => $nrPeriodoApuracao
            ]
        ];
        
        $response = $this->hcmServices->send('retrocederApuracao', $params, $this->hcmParameters->getNrorgTrabalho());
        
        return !empty($response['dataset']);
    }
    
    public function createApuracaoRequestAlertMessage($responseInfoDataset) {
        return array_reduce(
            $responseInfoDataset,
            function($carry, $item) {
                return $carry . $item['TEXTO'] . '<br>';
            },
            ''
        );
    }
} 